% This Matlab code enumerates set L_a(S) where S is a subset of {0,1}^4/a
% so that kappa(a)\{a} is a subset of S. See the manuscript entitled:
% The flip-graph of the 4-dimensional cube is connected (by L. Pournin)
% for comprehensive explanations regarding all the notations, and the
% problem at hand.
clear all;

disp('Which set do you want to enumerate?');
disp('La(S_1) -> 1');
disp('La(S_2) -> 2');
disp('La(S_3) -> 3');
Si=input ('1/2/3 [1]: ', 's');
if isempty(Si)
    Si = 1;
else
    Si=str2num(Si);
end

if(Si==1)
  vert2char={'b' 'c' 'd' 'e' 'f' 'g' 'i' 'j' 'k' 'l' 'm'};
  triangs_file='triangsS1.txt';
  circuits_file='circuitsS1.txt';
  pointconfiguration='{b,c,d,e,f,g,i,j,k,l,m}/a';
elseif(Si==2)
  vert2char={'b' 'c' 'e' 'f' 'g' 'h' 'i' 'l' 'm' 'n' 'o'};
  triangs_file='triangsS2.txt';
  circuits_file='circuitsS2.txt';
  pointconfiguration='{b,c,e,f,g,h,i,l,m,n,o}/a';
elseif(Si==3)
  vert2char={'b' 'c' 'd' 'e' 'h' 'i' 'l' 'm' 'n' 'o'};
  triangs_file='triangsS3.txt';
  circuits_file='circuitsS3.txt';
  pointconfiguration='{b,c,d,e,h,i,l,m,n,o}/a';
else
  disp('Please type in 1, 2, or 3!')
  finish
end

% reads the triangulations file into the cell array of strings triangs. This
% array is made up of one column. Each line in this array contains a string
% that defines one triangulation of set S. For example, the string
% 'bcej ceij' represents the triangulation of S that contains the two 
% tetrahedra {b,c,e,j}/a and {c,e,i,j}/a.

disp(['Loading triangulations of point configuration ' pointconfiguration ' from file ' triangs_file]);
tmp=textread(triangs_file,'%s','delimiter','\n','whitespace','');
for i=1:size(tmp,1)
  tri=regexprep(tmp{i,1}, '},{', ' ');
  for j=size(vert2char,2):-1:1
    tri=regexprep(tri, num2str(j-1), vert2char{1,j});
  end
  tri=regexprep(tri, ',', '');
  tri=regexprep(tri, '{{', '');
  triangs(i,1)={regexprep(tri, '}}', '')};
end
clear tmp;

% Reads the file containing the circuits of S into the cell array of
% strings tmp. This array is made up of one column. Each line in this
% array contains a string that defines one circuit of set S and its
% Radon partition. For example, the string 'bi j' represents circuit
% {b,i,j}/a whose Radon partition is {{b,i},{j}}/a.

disp(['Loading circuits of point configuration ' pointconfiguration ' from file ' circuits_file]);
tmp=textread(circuits_file,'%s','delimiter','\n','whitespace','');
tmp=tmp(3:size(tmp,1)-1,1);
for i=1:size(tmp,1)
  cir=regexprep(tmp{i,1}, '},{', ' ');
  for j=size(vert2char,2):-1:1
    cir=regexprep(cir, num2str(j-1), vert2char{1,j});
  end  
  cir=regexprep(cir, ',', '');
  cir=regexprep(cir, '[{', '');
  tmp(i,1)={regexprep(cir, '}]', '')}; 
end

% Reads the file containing the circuits of {0,1}^4 into the cell array
% of strings tmp2. This array is made up of one column. Each line in 
% this array contains a string that defines one circuit of {0,1}^4 and
% its Radon partition in the same format used above for the circuits of
% S. For example, the string 'bkn ejl' represents circuit {b,e,j,k,l,n}
% whose Radon partition is {{b,k,n},{e,j,l}}.

disp('Loading circuits of the 4-dimensional cube from file circuits4cube.txt');
tmp2=textread('circuits4cube.txt','%s','delimiter','\n','whitespace','');
tmp2=tmp2(3:size(tmp2,1)-1,1);
for i=1:size(tmp2,1)
  cir=regexprep(tmp2{i,1}, '},{', ' ');
  for j=16:-1:1
    cir=regexprep(cir, num2str(j-1), char(j+96));
  end  
  cir=regexprep(cir, ',', '');
  cir=regexprep(cir, '[{', '');
  tmp2(i,1)={regexprep(cir, '}]', '')}; 
end

% Finds the circuits of point configuration S that belong to Z(a)/a. These
% circuits are stored in the cell array of strings circuits, made up of one
% column. Each line in this array contains a string that defines one circuit
% of {0,1}^4/a, its Radon partition, and sets epsilon_a^-(z)/a and 
% epsilon_a^+(z)/a (see the manuscript for details). For example, the string
% 'fl dej' represents circuit {d,e,f,j,l}/a so that epsilon_a^-(z)/a={f,l}/a
% and epsilon_a^+(z)/a={d,e,j}/a. In addition, recall that for any such
% cicuit z/a, epsilon_a^-(z)/a and epsilon_a^+(z)/a are the two elements of
% the Radon partition of z/a. As a consequence, the Radon partition of the
% circuit represented by line 'fl dej' is indeed {{f,l},{d,e,j}}/a.

circuits={};
for i=1:size(tmp,1)
  [part1,part2]=strtok(tmp{i,1});
  part2=regexprep(part2, ' ', '');
  cir1=['a' part1 ' ' part2];
  cir2=['a' part2 ' ' part1];
  for j=1:size(tmp2,1)
    if(strcmp(cir1,tmp2{j,1}))
      circuits=[circuits;[part1 ' ' part2]];
    end
    if(strcmp(cir2,tmp2{j,1}))
      circuits=[circuits;[part2 ' ' part1]];        
    end
  end
end

% For all the circuits z/a stored in array circuits, computes the triangulation
% tau^-. This triangulation is the unique triangulation of z/a that contains
% epsilon_a^-(z)/a. See the manuscript for more details. These triangulations
% are stored in cell array tau. This array is made up of one column. Its i-th
% line contains a string that gives the triangulation tau^- of the circuit z
% stored in the i-th line of array circuits. For example, if line 3 of tau is
% string 'flej fldj flde' then the triangulation tau^- of the circuit z stored
% in line 3 of array circuits is the triangulation that contains {f,l,e,j}/a,
% {f,l,d,j}/a, and {f,l,d,e}/a as its three maximal faces. Observe that the
% vertices of each simplex in every triangulation are sorted in ascending
% alphabetical order, and that the list of the simplices in a triangulation is
% sorted in ascending lexicographic order.

disp('computing the triangulations tau^- for every z in Z(a)/a...');
tau={};
for i=1:size(circuits,1)
  % part1 and part2 are the two elements in the Radon partition
  % of circuit z found in the i-th line of array circuits.
  [part1,part2]=strtok(circuits{i,1});
  part2=regexprep(part2, ' ', '');
  sortedtri={};
  for j=1:size(part2,2)
    % removes the j-th letter of part2 and concatenates the result
    % with part1. This creates one of the simplices of tau^-
    % stored in the variable simplex.
    simplex=strcat(part1,regexprep(part2,part2(1,j),''));
    % the vertices of this simplex will be sorted in ascending
    % alphabetic order (this amounts to apply a permutation to the
    % characters in the string simplex).
    sortedsimplex={};
    % As a first step the vertices of the simplex are split into the
    % cell array sortedsimplex (one column per vertex, and one row) 
    for k=1:size(simplex,2)
      sortedsimplex=[sortedsimplex;simplex(1,k)];
    end
    % Sorts the cell array sortedsimplex in ascending alphabetic order
    sortedsimplex=sort(sortedsimplex);
    % Recreates the string simplex whose letters are now sorted.
    simplex='';
    for k=1:size(sortedsimplex,1)
      simplex=strcat(simplex,sortedsimplex{k,1});
    end
    % Puts the sorted simplex into the last row of cell array sortedtri
    sortedtri=[sortedtri;simplex];
  end
  % Sorts the cell array sortedtri in ascending lexicographic order
  sortedtri=sort(sortedtri);
  tri='';
  % Recreates a string tri with all the simplices of triangulation tau^-
  % sorted in ascendig lexicographic order and each separated of the next one
  % by a whitespace character.
  for k=1:size(sortedtri,1)
    tri=[tri sortedtri{k,1} ' '];
  end
  % Removes the last whitespace character in the string tri and stores it
  % into the last row of cell array tau.
  tau=[tau;tri(1:size(tri,2)-1)];
end

% Finds all the triangulations in La(S) and store them into the cell array
% La. This is done following  algorithm 1 found in the manuscript. Some
% variations from algorithm 1 have will be found whose only incidence is
% on the efficiency of the calculation. They will be mentioned below.

% Initializes La as an empty cell array.
La={};

% Index i points to the i-th triangulation of S, found in row i of cell
% array triangs.
for i=1:size(triangs,1)
   
   % String tri stores the i-th triangulation of S.
   tri=triangs{i,1};
   
   % Now the circuits in cell array circuits will be considered one by one.
   % The flag f1 will be equal to 1 if the circuit considered prevents the
   % triangulation tri to be in La, and to 0 otherwise. This flag is
   % initially set to 0, and the search will finish as soon as a circuit is
   % found that prevents triangulation tri to be in La. The first
   % difference with the algorithm given in the manuscript is that only one
   % flag is used here (whereas two flags f1 and f2 are used in the
   % manuscript for readability).
   f1=0;
   
   % Index j points to the j-th circuit, found in row j of cell array
   % circuits. It is initially equal to one and will be incremented in the
   % while loop below.
   j=1;

   % Here, the condition in the while loop will cause an exit either if all
   % the circuits have been tested or if a circuit is found that prevents
   % triangulation tri to be in La (this occurs when at the end of the while
   % loop flag f1 is equal to 1).
   while(j<=size(circuits,1) && f1==0)
     
     % string cir stores circuit j. 
     cir=circuits{j,1};
     
     % A preliminary check is performed to verify that all the vertices of
     % circuit cir are vertices of triangulation tri. Otherwise, the
     % circuit does not prevent the triangulation to be in La (because of
     % theorem 2 in the manuscript). The flag circuit_is_ok is initially
     % set to 1 and will be changed to 0 whenever a point of circuit cir
     % is not a vertex of triangulation tri. 
     circuit_is_ok=1;
     
     % Index k points to the k-th letter of circuit cir (which is also its 
     % k-th vertex). 
     for k=1:size(cir,2)
       if(isempty(strfind(tri, cir(k))))
         circuit_is_ok=0;
         break
       end
     end
    
     if(circuit_is_ok)
       
       % The first simplex in the triangulation tau^- of cir is stored in
       % the string simplex. The remainder of the triangulation is stored
       % in the string tmptau.
       [simplex,tmptau]=strtok(tau{j,1});
       
       % Now, it will be checked whether the simplex stored in the string
       % simplex is an element of the triangulation tri (equivalently, that
       % all the characters of string simplex are found in a same tetrahedron
       % in string tri). If simplex is an element of tri, its link in tri
       % is computed simulmtaneously and stored in the string link1.       
       link1='';

       % The following while loop will consider one by one all the tetrahadra
       % of triangulation tri. They will be removed from the string tmptri
       % (initially equal to string tri) using Matlab command strtok and
       % stored in the string tetra. 
       tmptri=tri;
       while(isempty(tmptri)==0)
         [tetra,tmptri]=strtok(tmptri);
         tmplink=tetra;
         % This for loop removes every character of the string simplex from
         % the string tmplink.
         for k=1:size(simplex,2)
           tmplink=regexprep(tmplink, simplex(1,k),'');
         end
         
         % If the number of characters removed from string tmplink is equal
         % to the number of characters in the string simplex then this simplex
         % is a subset of tetrahadron tetra, and the remaining of tmplink is
         % an element of its link in triangulation tri. In this case tmplink
         % is concatenated with link1 and flag f1 is set to 1. This flag can
         % still be changed to 0 in the while loop below if some other maximal
         % simplex in triangulation tau^- of cir is not found in the triangulation,
         % or if its link in triangulation tri is not equal to link1.
         if(size(simplex,2)+size(tmplink,2)==4)
           link1=[link1 tmplink ' '];
           f1=1;
         end
       end
      
       % The remaining simplices in tmptau are now checked in a while loop.
       % Observe that this while loop is only executed if f1=1 (that is, if
       % the first simplex of tau^- of cir was found in triangulation tri).
       % For each of these simplices, stored in the string simplex, it is
       % checked that this simplex is found in triangulation tri and that
       % its link in tri is equal to link1. If one of these two conditions
       % are not satisfied, then f1 will be set to 0 and the loop will
       % terminate. The loop alternatively terminates when all the
       % simplices in triangulation tau^- of cir have been found in
       % triangulation tri, and admit link1 as their link in tri. In this
       % case, circuit cir prevents triangulation tri to be in La.
       while(isempty(tmptau)==0 && f1==1)
         [simplex,tmptau]=strtok(tmptau);
         
         % String link2 will store the link of simplex is this simplex is
         % found in triangulation tri. It will be later compared to link1.
         link2='';
         
         % The following while loop will consider one by one all the tetrahadra
         % of triangulation tri. They will be removed from the string tmptri
         % (initially equal to string tri) using Matlab command strtok and
         % stored in the string tetra.
         tmptri=tri;

         % Flag f1 is reset to 0 and will be set to 1 in the for loop if
         %simplex is found as a subset of an element of triangulation tri.
         f1=0;
         while(isempty(tmptri)==0)
           [tetra,tmptri]=strtok(tmptri);
           tmplink=tetra;
           
           % This for loop removes every character in the string simplex from
           % the string tmplink.
           for k=1:size(simplex,2)
             tmplink=regexprep(tmplink, simplex(1,k),'');
           end

           % Again, if the number of characters removed from string tmplink is equal
           % to the number of characters in the string simplex then this simplex
           % is a subset of tetrahadron tetra, and the remaining of tmplink is
           % an element of its link in triangulation tri. In this case tmplink
           % is concatenated with link2.
           if(size(simplex,2)+size(tmplink,2)==4)
             link2=[link2 tmplink ' '];
             f1=1;
           end
         end

         % If the two strings link1 and link2 are not equal then circuit
         % cir is not flippable in triangulation tri and it does not
         % prevent trias triangulation to belong to La. Flag f1 is
         % therefore set to 0, and another circuit will be considered.
         if(strcmp(link1,link2)==0)
           f1=0;
         end
       end

       % If a circuit is found that prevents triangulation tri to be in La
       % then this circuit is stored in circuit_found to be displayed
       % later.
       if(f1==1)
         circuit_found=cir;
       end
     end
     
     % Increases the index j of the circuit
     j=j+1;
   end

   % If f1=0 then no circuit was found that prevents triangulation tri to
   % be in La. So tri is stored in the last row of cell array La. 
   if(f1==0)
     disp([num2str(i) ' over ' num2str(size(triangs,1)) ': triangulation ' triangs{i,1} '/a belongs to La(S)'])
     La=[La;tri];  
   else
     disp([num2str(i) ' over ' num2str(size(triangs,1)) ': triangulation ' triangs{i,1} '/a is not in La(S) because of circuit ' circuit_found '/a.' ]);
   end
end

% Finally displays set La and its size.
disp(' ');
disp(['Found the following ' num2str(size(La,1)) ' triangulations in La(S) :']);
for i=1:size(La,1)
  disp([num2str(i) ': ' La{i,1} '/a' ]);
end


